package de.unikoblenz.fgbks.core.dmn.domain.vdmn;

import de.unikoblenz.fgbks.base.domain.Description;
import de.unikoblenz.fgbks.base.domain.StringValue;
import de.unikoblenz.fgbks.base.domain.Text;
import de.unikoblenz.fgbks.core.dmn.domain.common.RowNumber;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionTableId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.DefinitionId;
import de.unikoblenz.fgbks.core.dmn.domain.ids.RuleId;
import java.util.List;
import java.util.Optional;

/**
 * This interface represents a single value (or cell) in a {@link VDmnDecisionTable}. It can either
 * be a {@link VDmnInputValue} (in a {@link VDmnInputColumn}) or a {@link VDmnOutputValue} (in a
 * {@link VDmnOutputColumn}). Furthermore each value exists in one {@link VDmnRule}.
 */
public interface VDmnValue extends VDmnElement {

  /**
   * Get the {@link VDmnRule} of the value.
   *
   * @return the {@link VDmnRule}
   */
  VDmnRule getVDmnRule();

  /**
   * Get the original text of the value.
   *
   * @return the {@link Text}
   */
  Text getText();

  /**
   * Get the description, if present.
   *
   * @return the {@link Description} as optional
   */
  Optional<Description> getDescription();

  /**
   * Get the column of the value. Can be a {@link VDmnInputColumn} or a {@link VDmnOutputColumn}.
   *
   * @return the {@link VDmnColumn}
   */
  VDmnColumn getVDmnColumn();

  /**
   * Get the possible string values, which are defined in {@link VDmnColumn#getPredefinedValues()}.
   *
   * @return an optional list of string values.
   */
  default Optional<List<StringValue>> getPossiblePredefinedValues() {
    return getVDmnColumn().getPredefinedValues();
  }

  /**
   * Get the {@link DefinitionId} of the {@link VDmnDefinitions}.
   *
   * @return the {@link DefinitionId}.
   */
  default DefinitionId getDefinitionId() {
    return getVDmnDefinition().getDefinitionId();
  }

  /**
   * Get the {@link DecisionId} of the {@link VDmnDecision}.
   *
   * @return the {@link DecisionId}.
   */
  default DecisionId getDecisionId() {
    return getVDmnDecision().getDecisionId();
  }

  /**
   * Get the {@link DecisionTableId} of the {@link VDmnDecisionTable}.
   *
   * @return the {@link DecisionTableId}.
   */
  default DecisionTableId getDecisionTableId() {
    return getVDmnDecisionTable().getDecisionTableId();
  }

  /**
   * Get the {@link VTypeRef} of the {@link VDmnColumn}.
   *
   * @return the {@link VTypeRef}.
   */
  default VTypeRef getTypeRef() {
    return getVDmnColumn().getTypeRef();
  }

  /**
   * Get the {@link RuleId} of the {@link VDmnRule}.
   *
   * @return the {@link RuleId}.
   */
  default RuleId getRuleId() {
    return getVDmnRule().getRuleId();
  }

  /**
   * Get the {@link RowNumber} of the {@link VDmnRule}.
   *
   * @return the {@link RowNumber}.
   */
  default RowNumber getRowNumber() {
    return getVDmnRule().getRowNumber();
  }

  /**
   * Get a boolean, which determines, if the value is a input value.
   *
   * @return true, if the value is a input value
   */
  default boolean isInputValue() {
    return getVDmnColumn().isInputColumn();
  }

  /**
   * Get a boolean, which determines, if the value is a output value.
   *
   * @return true, if the value is a output value
   */
  default boolean isOutputValue() {
    return getVDmnColumn().isOutputColumn();
  }

  /**
   * Get the {@link VDmnDecisionTable}.
   *
   * @return the {@link VDmnDecisionTable}.
   */
  default VDmnDecisionTable getVDmnDecisionTable() {
    return getVDmnColumn().getVDmnDecisionTable();
  }

  /**
   * Get the {@link VDmnDecision}.
   *
   * @return the {@link VDmnDecision}.
   */
  default VDmnDecision getVDmnDecision() {
    return getVDmnDecisionTable().getVDmnDecision();
  }

  /**
   * Get the {@link VDmnDefinitions}.
   *
   * @return the {@link VDmnDefinitions}.
   */
  default VDmnDefinitions getVDmnDefinition() {
    return getVDmnDecision().getVDmnDefinition();
  }
}
