package de.unikoblenz.fgbks.core.dmn.verification.verifier.impl;

import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.getColumnStringName;
import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.getDecisionStringName;
import static de.unikoblenz.fgbks.core.dmn.domain.vdmn.utils.VDmnFunctions.templateDecision;
import static de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix.SHOW_INPUT_COLUMNS;

import de.unikoblenz.fgbks.base.domain.Name;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputData;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnNode;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntry.VerificationClassification;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryElement;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.Action;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionScope;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.ActionType;
import de.unikoblenz.fgbks.core.dmn.verification.result.actions.VerificationFix;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.AbstractVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.DmnVerifier;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.MissingInputDataVerification;
import java.util.List;
import java.util.stream.Collectors;

@DmnVerifier(verifierType = MissingInputDataVerification.class)
public class MissingInputDataVerifier extends AbstractVerifier {

  @Override
  protected void doVerification() {
    for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getVDmnDecisions()) {
      List<VDmnNode> inputNodes = decision.getVDmnInformationRequirements();
      // Check for each input column if there is one input data node or one output column from a
      // decision node
      for (VDmnInputColumn inputColumn : decision.getVDmnDecisionTable().getVDmnInputColumns()) {
        checkExistingInputData(inputColumn, inputNodes);
      }
    }
  }

  private void checkExistingInputData(VDmnInputColumn inputColumn, List<VDmnNode> inputNodes) {
    // if the inputColumn has no name, add the column
    if (inputColumn.getName().isPresent()) {
      Name inputColumnName = inputColumn.getName().get();
      // check, if there is one input Data with the same name
      for (VDmnNode node : inputNodes) {
        // input Data node, check only the name
        if (node.isInputData()) {
          if (node.getName().isPresent() && node.getName().get().equals(inputColumnName)) {
            return;
          }
        } else if (node.isDecision()) {
          VDmnDecision decision = (VDmnDecision) node;
          // check for each output column, if the name of the output column matches the input column
          // name
          for (VDmnOutputColumn outputColumn :
              decision.getVDmnDecisionTable().getVDmnOutputColumns()) {
            if (outputColumn.getName().isPresent()
                && outputColumn.getName().get().equals(inputColumnName)) {
              return;
            }
          }
        }
      }
    }
    // no input data found yet -> add column to results
    vreFactory
        .addElement(VerificationResultEntryElement.create(inputColumn))
        .addVerificationFix(SHOW_INPUT_COLUMNS);
    // Action for creating input Data
    if (inputColumn.getName().isPresent()) {
      vreFactory
          .addVerificationFix(
              VerificationFix.getBuilder()
                  .withFixName(new Name("New Data Input"))
                  .addAction(
                      Action.getBuilder()
                          .withActionType(ActionType.CREATE)
                          .withActionScope(ActionScope.INPUT_DATA)
                          .addIdValue(inputColumn.getVDmnDecision().getDecisionId())
                          .addValue("name", inputColumn.getName().get().getValue())
                          .build())
                  .build())
          // Action for deleting column
          .addVerificationFix(
              VerificationFix.getBuilder()
                  .withFixName(new Name("Delete Column"))
                  .addAction(
                      Action.getBuilder()
                          .withActionType(ActionType.DELETE)
                          .withActionScope(ActionScope.INPUT_COLUMN)
                          .addIdValue(inputColumn.getVDmnDecision().getDecisionId())
                          .addIdValue(inputColumn.getInputId())
                          .build())
                  .build());
      // Actions for connecting to existing input Data
      searchPossibleInputData(inputColumn);
      // Actions for connection to existing decisions
      searchPossibleInputDecisions(inputColumn);
      // Actions for creating output columns in connected decisions
      searchPossibleConnectedDecisions(inputColumn);
    }
    // Add to result
    vreFactory.addToEntry(
        VerificationClassification.WARNING,
        templateDecision(inputColumn.getVDmnDecision())
            + "Input column \"%s\" has no input data node.",
        getColumnStringName(inputColumn));
  }

  private void searchPossibleInputData(VDmnInputColumn inputColumn) {
    for (VDmnInputData inputData : dmnObjectContainer.getVDmnDefinition().getVDmnInputData()) {
      if (inputData.getName().isPresent()
          && inputData.getName().get().equals(inputColumn.getName().get())) {
        vreFactory.addVerificationFix(
            VerificationFix.getBuilder()
                .withFixName(new Name("Connect to Input Data " + inputData.getName().get()))
                .addAction(
                    Action.getBuilder()
                        .withActionType(ActionType.UPDATE)
                        .withActionScope(ActionScope.INPUT_DATA)
                        .addIdValue(inputColumn.getVDmnDecision().getDecisionId())
                        .addIdValue(inputData.getInputDataId())
                        .build())
                .build());
      }
    }
  }

  private void searchPossibleInputDecisions(VDmnInputColumn inputColumn) {
    for (VDmnDecision decision : dmnObjectContainer.getVDmnDefinition().getVDmnDecisions()) {
      for (VDmnOutputColumn outputColumn :
          decision.getVDmnDecisionTable().getVDmnOutputColumns().stream()
              .filter(
                  c ->
                      !c.getVDmnDecision()
                          .getDecisionId()
                          .equals(inputColumn.getVDmnDecision().getDecisionId())
                          && c.getName().isPresent()
                          && c.getName().get().equals(inputColumn.getName().get())
                          && inputColumn.getTypeRef() == c.getTypeRef())
              .collect(Collectors.toList())) {
        vreFactory.addVerificationFix(
            VerificationFix.getBuilder()
                .withFixName(
                    new Name(
                        "Connect to Decision "
                            + getDecisionStringName(outputColumn.getVDmnDecision())))
                .addAction(
                    Action.getBuilder()
                        .withActionType(ActionType.UPDATE)
                        .withActionScope(ActionScope.DECISION)
                        .addIdValue(outputColumn.getVDmnDecision().getDecisionId())
                        .addValue(
                            "targetDecisionId",
                            inputColumn.getVDmnDecision().getDecisionId().getValue())
                        .build())
                .build());
      }
    }
  }

  private void searchPossibleConnectedDecisions(VDmnInputColumn inputColumn) {
    for (VDmnNode node : inputColumn.getVDmnDecision().getVDmnInformationRequirements()) {
      if (node.isDecision()) {
        VDmnDecision decision = (VDmnDecision) node;
        vreFactory.addVerificationFix(
            VerificationFix.getBuilder()
                .withFixName("New Output Column in decision " + getDecisionStringName(decision))
                .addAction(
                    Action.getBuilder()
                        .withActionType(ActionType.CREATE)
                        .withActionScope(ActionScope.OUTPUT_COLUMN)
                        .addIdValue(decision.getDecisionId())
                        .addValue("name", inputColumn.getName().get().getValue())
                        .addValue("typeRef", inputColumn.getTypeRef().getName())
                        .build())
                .build());
      }
    }
  }
}
